@@ -10,7 +10,6 @@ import redis
 from collections import deque
 import polars
 import numpy as np
-import s3fs
 import gc
 from pyarrow.fs import S3FileSystem, LocalFileSystem
 from pyarrow.dataset import FileSystemDataset, ParquetFileFormat
@@ -62,13 +61,13 @@ class InputEC2ParquetDataset:
 
     def get_next_batch(self, mapper_id, pos=None):
 
-        s3fs = S3FileSystem()
+        s3 = S3FileSystem()
 
         def download(file):
             with warnings.catch_warnings():
                 warnings.simplefilter("ignore")
                 return polars.from_arrow(pq.read_table(self.bucket + "/" +
-                              file, columns=self.columns, filters=self.filters, use_threads= False, use_legacy_dataset = True, filesystem = s3fs))
+                              file, columns=self.columns, filters=self.filters, use_threads= False, use_legacy_dataset = True, filesystem = s3))
 
         assert self.num_channels is not None
         executor = concurrent.futures.ThreadPoolExecutor(max_workers=self.workers)
